@@ -9,6 +9,7 @@ from amfast.class_def import ClassDefMapper
 import connection_manager as cm
 import subscription_manager as sm
 import flex_messages as messaging
+import thread_pool
 
 class ChannelError(amfast.AmFastError):
     pass
@@ -127,19 +128,23 @@ class HttpChannel(Channel):
 
     attributes
     ===========
-     * wait_interval - int, Number of seconds to wait before sending response to client
-         when a polling request is received. Set to -1 to configure channel as a
-         long-polling channel.
+     * wait_interval - int, Number of millisecondsseconds to wait before sending
+         response to client when a polling request is received. Set to -1 to
+         configure channel as a long-polling channel. Default = 0
+     * poll_interval - int, Number of milliseconds between message polling operations
+         when channel_set.notify == False
     """
 
     # Content type for amf messages
     CONTENT_TYPE = 'application/x-amf'
 
-    def __init__(self, name, max_connections=-1, endpoint=None, wait_interval=0):
+    def __init__(self, name, max_connections=-1, endpoint=None,
+        wait_interval=0, poll_interval=500):
 
         Channel.__init__(self, name, max_connections, endpoint)
 
         self.wait_interval = wait_interval
+        self.poll_interval = poll_interval
 
     def getBadMethodMsg(self):
         return "405 Method Not Allowed\n\nAMF request must use 'POST' method."
@@ -157,21 +162,68 @@ class HttpChannel(Channel):
         return "500 Internal Server Error\n\nAmFast server error.%s" % msg
 
     def waitForMessage(self, packet, message, connection):
-        """Waits for a new message.
+        """Waits until a new message is published to this connection.
+        Returns list of messages.
+        """
+
+        if self.channel_set.notify_connections is True:
+            return self._waitForMessage(packet, message, connection)
+        else:
+            return self._pollForMessage(packet, message, connection)
+
+    def _waitForMessage(self, packet, message, connection):
+        """Waits until notified that a new message is available, then returns messages.
 
         This is blocking, and should only be used
         for Channels where each connection is a thread.
+
+        Synchronous servers should override this method.
         """
+
         event = threading.Event()
         connection.setNotifyFunc(event.set)
         
         if (self.wait_interval > -1):
-            event.wait(self.wait_interval)
+            event.wait(float(self.wait_interval) / 1000)
 
         # Event.set has been called,
         # or timeout has been reached.
         connection.unSetNotifyFunc()
 
+        return self.channel_set.subscription_manager.pollConnection(connection)
+
+    def _pollForMessage(self, packet, message, connection):
+        """Repeatedly polls for a new message until
+        message is available or wait_interval is reached.
+
+        This is blocking, and should only be used
+        for Channels where each connection is a thread.
+
+        Synchronous servers should override this method.
+        """
+        # If True, don't store persistent 'last_polled' value every poll operation.
+        soft_touch = hasattr(self.channel_set.connection_manager, "softTouchPolled")
+
+        total_time = 0
+        poll_secs = float(self.poll_interval) / 1000
+        wait_secs = float(self.wait_interval) / 1000
+        while True:
+            event = threading.Event()
+            event.wait(poll_secs)
+            msgs = self.channel_set.subscription_manager.pollConnection(connection, soft_touch)
+            if len(msgs) > 0:
+                if soft_touch is True:
+                    # Store 'last_polled' value.
+                    connection.touchPolled()            
+                return msgs
+            
+            total_time += poll_secs
+            if total_time > wait_secs or connection.connected is False:
+                if soft_touch is True:
+                    # Store 'last_polled' value.
+                    connection.touchPolled()
+                return ()
+
 class ChannelSet(object):
     """A collection of Channels.
 
@@ -258,29 +310,40 @@ class ChannelSet(object):
         self.subscription_manager.deleteConnection(connection)
         self.connection_manager.deleteConnection(connection)
 
-    def scheduleClean(self, dummy=True):
+    def scheduleClean(self):
         """Schedule connection cleaning procedure to run sometime in the future."""
 
-        if dummy is True:
+        if amfast.use_dummy_threading is True:
             amfast.logger.warn('Connection cleaning was NOT scheduled.')
             return
 
+        thread = threading.Timer(self.clean_freq, None)
+        thread.daemon = True
         def _clean():
             self.clean()
-            self.scheduleClean()
-
-        thread = threading.Timer(self.clean_freq, _clean)
-        thread.daemon = True
+            thread.start()
+        thread.function = _clean
         thread.start()
 
     def clean(self):
         """Clean out expired connections."""
-        amfast.logger.debug("Cleaning connections.")
+        if amfast.log_debug is True:
+            amfast.logger.debug("Cleaning channel.")
 
         current_time = time.time() * 1000
         for connection_id in self.connection_manager.iterConnectionIds():
             self.cleanConnection(connection_id, current_time)
 
+        if hasattr(self.subscription_manager, 'deleteExpiredMessages'):
+            # TODO: better interface for deleting expired messages.
+            #
+            # Perhaps iterate through messages and delete expired
+            # similar to how connection cleaning works??
+            #
+            # Currently, some subscriptions managers auto-delete
+            # expired messages, while others require a method to be called.
+            self.subscription_manager.deleteExpiredMessages(current_time)
+
     def cleanConnection(self, connection_id, current_time): 
         if amfast.log_debug is True: 
             amfast.logger.debug("Cleaning connection: %s" % connection_id) 
@@ -345,11 +408,11 @@ class ChannelSet(object):
 
         arguments:
         ===========
-         * msg - AbstractMessage, the Flex message that was published.
+         * topic - string, topic to notify
+         * sub_topic - string, sub_topic to notify
         """
-
-        thread = threading.Thread(target=self._notifyConnections, args=(topic, sub_topic))
-        thread.run()
+       
+        thread_pool.GlobalThreadPool().addTask(self._notifyConnections, args=(topic, sub_topic)) 
 
     def _notifyConnections(self, topic, sub_topic):
         """Do the real work of notifyConnections."""
