@@ -20,39 +20,44 @@ class TwistedChannelSet(ChannelSet):
         return channel.render_POST(request)
 
     def scheduleClean(self):
-        reactor.callLater(self.clean_freq, self.clean)
+        cleaner = task.LoopingCall(self.clean)
+        cleaner.start(self.clean_freq, False)
 
     def notifyConnections(self, topic, sub_topic):
-        reactor.callLater(0.1, self._notifyConnections, topic, sub_topic)
+        reactor.callLater(0, self._notifyConnections, topic, sub_topic)
 
     def _notifyConnections(self, topic, sub_topic):
-        for connection_id in self.subscription_manager.iterSubscribers(topic, sub_topic):
+        iter = self.subscription_manager.iterSubscribers(topic, sub_topic)
+        def _notify():
             try:
-                connection = self.connection_manager.getConnection(connection_id, False)
-            except cm.NotConnectedError:
-                continue
-
-            if connection.notify_func is not None:
-                reactor.callLater(0.1, connection.notify_func)
+                connection_id = iter.next()
+                try:
+                    connection = self.connection_manager.getConnection(connection_id, False)
+                except cm.NotConnectedError:
+                    pass
+                else:
+                    if connection.notify_func is not None:
+                        reactor.callLater(0, connection.notify_func)
+                reactor.callLater(0, _notify)
+            except StopIteration:
+                pass
+        reactor.callLater(0, _notify)
 
     def clean(self):
-        amfast.logger.debug("Cleaning connections.")
+        if amfast.log_debug is True:
+            amfast.logger.debug("Cleaning channel.")
 
         current_time = time.time()
-        deferreds = []
-        for connection_id in self.connection_manager.iterConnectionIds():
-            d = defer.Deferred()
-            d.addCallbacks(self.cleanConnection, callbackArgs=(current_time,))
-            reactor.callLater(0.3, d.callback, connection_id)
+        iter = self.connection_manager.iterConnectionIds()
 
-        if len(deferreds) > 0:
-            def _scheduleClean(val):
-                self.scheduleClean()
-
-            deferredList = defer.DeferredList(deferreds)
-            deferredList.addCallback(self.scheduleClean)
-        else:
-            self.scheduleClean()
+        def _clean():
+            try:
+                connection_id = iter.next()
+                reactor.callLater(0, self.cleanConnection, connection_id, current_time)
+                reactor.callLater(0, _clean)
+            except StopIteration:
+                pass
+        reactor.callLater(0, _clean)
 
 class TwistedChannel(Resource, HttpChannel):
     """An AMF messaging channel that can be used with Twisted Web."""
@@ -67,11 +72,10 @@ class TwistedChannel(Resource, HttpChannel):
     # so that it can be available to targets.
     TWISTED_REQUEST = '_twisted_request'
 
-    def __init__(self, name, max_connections=-1, endpoint=None,
-        timeout=1200, wait_interval=0):
+    def __init__(self, *args, **kwargs):
 
         Resource.__init__(self)
-        HttpChannel.__init__(self, name, max_connections, endpoint, wait_interval)
+        HttpChannel.__init__(self, *args, **kwargs)
 
     def render_POST(self, request):
         """Process an incoming AMF packet."""
@@ -83,7 +87,6 @@ class TwistedChannel(Resource, HttpChannel):
             d.addCallbacks(self.checkComplete, self.fail, callbackArgs=(request,), errbackArgs=(request,))
             d.callback(int(request.getHeader('Content-Length')))
 
-            request.setHeader('Content-Type', self.CONTENT_TYPE)
             return server.NOT_DONE_YET
 
     def decode(self, raw_request, request):
@@ -160,6 +163,7 @@ class TwistedChannel(Resource, HttpChannel):
 
     def finish(self, raw_response, request):
         """Send response to client when message is complete."""
+        request.setHeader('Content-Type', self.CONTENT_TYPE)
         request.write(raw_response)
         request.finish()
 
@@ -167,8 +171,9 @@ class TwistedChannel(Resource, HttpChannel):
         request.setResponseCode(500, message)
         self.finish(message, request)
 
-    def waitForMessage(self, packet, message, connection):
-        """Overridden to be non-blocking."""
+    def setupPollRequest(self, packet, connection):
+        """Setup a request for a long-poll operation."""
+
         # Set flag so self.finish
         # does not get called when
         # response packet is returned 
@@ -177,45 +182,99 @@ class TwistedChannel(Resource, HttpChannel):
 
         request = getattr(packet, self.TWISTED_REQUEST)
 
-        # Remove notify function if client drops connection.
-        _connectionLost = request.connectionLost
-        def _connection_lost(reason):
-            connection.unSetNotifyFunc()
-            _connectionLost(reason)
-            request.finish()
-        request.connectionLost = _connection_lost
+        return request
+
+    def finishPoll(self, request, packet, message, messages):
+        """Finish a request that has been waiting for messages."""
+
+        if request.finished:
+            # Someone else has already called this function,
+            # or twisted has finished the request for some other reason.
+            return
+
+        if isinstance(self.endpoint, AmfEndpoint):
+            # Make sure messages are not encoded as an ArrayCollection
+            messages = AsNoProxy(messages)
+        message.response_msg.body.body = messages
+ 
+        delattr(packet.response, self.MSG_NOT_COMPLETE)
+        self.checkComplete(packet.response, request)
+
+    def _waitForMessage(self, packet, message, connection):
+        """Overridden to be non-blocking."""
+
+        request = self.setupPollRequest(packet, connection)
 
-        # This function gets called when a message is published,
-        # or wait_interval is reached.
         timeout_call = None
         def _notify():
-            if request.finished:
-                # Someone else has already called this function,
-                # or twisted has finished the request for some other reason.
-                return
-
+            # This function gets called when a message is published,
+            # or wait_interval is reached.
             if timeout_call is not None and timeout_call.active():
+                # Disable time out callback
                 timeout_call.cancel()
-            
+
+            # Disable notify function.
             connection.unSetNotifyFunc()
 
             # Get messages and add them
             # to the response message
             messages = self.channel_set.subscription_manager.pollConnection(connection)
-            
-            if isinstance(packet.channel.endpoint, AmfEndpoint):
-                # Make sure messages are not encoded as an ArrayCollection
-                messages = AsNoProxy(messages)
-            message.response_msg.body.body = messages
- 
-            delattr(packet.response, self.MSG_NOT_COMPLETE)
-            self.checkComplete(packet.response, request)
-
+            self.finishPoll(request, packet, message, messages)
         connection.setNotifyFunc(_notify)
 
         # Notify when wait_interval is reached
         if self.wait_interval > -1:
-            timeout_call = reactor.callLater(self.wait_interval, _notify)
+            timeout_call = reactor.callLater(float(self.wait_interval) / 1000, _notify)
+
+        # Cleanup if client drops connection.
+        _connectionLost = request.connectionLost
+        def _connection_lost(reason):
+            _connectionLost(reason)
+            _notify()
+        request.connectionLost = _connection_lost
+
+        return ()
+
+    def _pollForMessage(self, packet, message, connection):
+        """Overridden to be non-blocking."""
+
+        request = self.setupPollRequest(packet, connection)
+
+        poller = task.LoopingCall(None)
+        
+        def _timeout():
+            poller.stop()
+            messages = self.channel_set.subscription_manager.pollConnection(connection)
+            self.finishPoll(request, packet, message, messages)
+
+        # Cleanup if client drops connection.
+        _connectionLost = request.connectionLost
+        def _connection_lost(reason):
+            _connectionLost(reason)
+            _timeout()
+        request.connectionLost = _connection_lost
+
+        if self.wait_interval > -1:
+            timeout_call = reactor.callLater(float(self.wait_interval) / 1000, _timeout)
+        else:
+            timeout_call = None
+
+        def _poll():
+            messages = self.channel_set.subscription_manager.pollConnection(connection)
+            if len(messages) > 0:
+                poller.stop()
+                if timeout_call is not None and timeout_call.active():
+                    # Disable time out callback
+                    timeout_call.cancel()
+
+                self.finishPoll(request, packet, message, messages)
+            elif connection.connected is False:
+                _timeout()
+
+        poller.f = _poll
+        poller.start(float(self.poll_interval) / 1000)
+
+        return ()
 
 class StreamingTwistedChannel(TwistedChannel):
     """Handles streaming http connections."""
@@ -250,8 +309,15 @@ class StreamingTwistedChannel(TwistedChannel):
 
     def startStream(self, msg, request):
         """Get this stream rolling!"""
+
         connection = self.channel_set.connection_manager.getConnection(msg.headers.get(msg.FLEX_CLIENT_ID_HEADER))
 
+        if self.channel_set.notify_connections is True:
+            poller = None
+        else:
+            # Repeatedly poll for messages.
+            poller = task.LoopingCall(None)
+
         # Remove notify function if client drops connection.
         _connectionLost = request.connectionLost
         def _connection_lost(reason):
@@ -263,7 +329,9 @@ class StreamingTwistedChannel(TwistedChannel):
         # This function gets called when a message is published.
         def _notify():
             if connection.connected is False:
-                # Connection is disconnected
+                if poller is not None:
+                    poller.stop()
+
                 connection.unSetNotifyFunc()
                 msg = messaging.StreamingMessage.getDisconnectMsg()
                 request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
@@ -275,6 +343,10 @@ class StreamingTwistedChannel(TwistedChannel):
                 request.write(messaging.StreamingMessage.prepareMsg(msg, self.endpoint))
         connection.setNotifyFunc(_notify)
 
+        if poller is not None:
+            poller.f = _notify
+            poller.start(float(self.poll_interval) / 1000, False) 
+
         # Acknowledge connection
         response = msg.acknowledge()
         response.body = connection.id
@@ -302,4 +374,4 @@ class StreamingTwistedChannel(TwistedChannel):
             request.write(chr(messaging.StreamingMessage.NULL_BYTE))
 
         looper.f = _beat
-        looper.start(self.heart_interval)
+        looper.start(self.heart_interval, False)
